/*
 * comptime.c (wmmuezzin)
 * par lilian
 *
 * 1999/Paris/La nuit
 *
 * Tres largement inspire (voir meme pompe) du programme "praytime".
 * Merci a : Waleed A. Muhanna
 *
 */

/*
 * Les includes
 */

#include        <sys/types.h>
#include        <math.h>
#include        <stdio.h>
#include        <time.h>
#include        <stdlib.h>
/*read config file*/
#include	<string.h>
#include	<errno.h>
#include	<stdarg.h>
/*
 * Les definitions
 */

#define MAXNAMEL	40
#define DPR		(57.29577951308230876799)
#define RPD		(0.01745329251994329577)
#define HPR		(3.81971863420548805845)
#define FABS(a)         ((a) > 0 ? (a) : -(a))
#define FMOD(a,b)       ((a) - floor( (a)/(b) ) * (b))
#define MAXENV		9
#define TIMZ		3.0
#define MINAGE		13.5
#define SUNSET		19.5 
#define TIMDIF		(SUNSET-MINAGE) 
#define NMONTHS		(1405*12+1)
#define NULLP		((char *)0)

typedef struct sdate {
        float time;
        int day;
        int mon;
        int year;
        int dw;
        double nmtime;
} SDATE;

char    	dir[4] = {'E', 'W', 'N', 'S'};
short   	leap, beginDayLight, endDayLight; 
short   	latd, latm, longd, longm, latIsS, longIsW, fajrByInterval, fajrByMaxInterval;
float   	tim[366][8];
double  	asrShadowRatio, fajrInterval, fajrDepr;
double  	cosobl, sinobl, perigee, dmanom, anom0;
double  	c1, c2, delsid, sidtm0;
double  	latitude,  longitude, timeZone;
short   	hasDayLt;
short   	hflag=0; 
int     	stdow;
SDATE		hd;

extern int	list_praying_hour[6];
extern int	list_praying_minute[6];
extern short	qiblah_d, qiblah_m;
extern int 	reminder_minutes;

/*read config file -zul 07-2003*/
/*codes lifted of wmtz         */
extern char scriptFileAzan[253]; 
extern char scriptFileOpen[253];
extern char scriptFileRemind[253];

char          *config_file;
double        longitude = 0.0;
double        latitude = 0.0;
char   *longc;
char   *latc, clatc[2];
char   *dlc;
char   *pname;
char	*msg;
/*
 * Les fonctions
 */

void	GetCallingPrayer(void);
void	MakeSchedule(short day, short month, short year);
void    ComputeHours(short year, short first, short last);
short	DayNum(short d, short m, short y);
void	SetDow(short y, short m, short d);
void	HSchedule(short m, short y);
void    ComputeConstants(short year);
void    Qiblah(short day, short month, short year);
double  NoonTime(short nday, double* coaltn);
double  Tempus(short nday, double coalt, double time0);
void	DayLight(short year, short* leap, short hasDayLt, short* begin, short* finish);
void    ShowError(char *s);
void    SetPrayTimeList(short first, short last, short startdate, short cd, short cm);
double  Degree2Radian(double degree);
double  DegreeMinute2Degree( short degree, short minute);
double  DegreeMinuteSecond2Degree(long degree, short min, double sec);
void    Degree2DegreeMinute( double degree, short *deg, short *min);
double	HourMinuteSecond2Hour( short hour, short min, double sec);
double	MoonPhase(long  n, int nph);
double	Visible(long n, double *rjd);
double	JulianDay( int year, int month, int day, float time);
int	NDays(int m, int y);
SDATE	*CalculDate(double julday );
SDATE	*HDate (int y, int m, int d);
SDATE	*GDate( int y, int m, int d);

/*read config file */
int    ReadConfigInt(FILE *fp, char *setting, int *value);
int    ReadConfigString(FILE *fp, char *setting, char *value);
int    Read_Config_File(char *filename);
void   errH(int printErrno, int exitCode, const char *msg, ...);

/*
 * Yalla 2 !!
 */

void GetCallingPrayer() {

	short   day=0, month=0, year;
	extern double atof(); 
	extern time_t time();
	struct tm *tm;
	time_t t;

	fajrByMaxInterval = 0; fajrByInterval = 0; fajrInterval = 1.5;
	fajrDepr = 18.0; asrShadowRatio = 1.0;

	t = time((time_t *)0);
	tm = localtime(&t);
	day = tm->tm_mday;
	month = tm->tm_mon+1;
	year = 1900 + tm->tm_year;
	
	/*trying to read config file: zul 07-2003*/
	char	*home;
    home = getenv("HOME");
	config_file = malloc( strlen(home) + 22 );
    if (config_file == NULL)
        errH(1, 1, "malloc");
        sprintf(config_file, "%s/.wmsolat/wmsolatrc", home);
		/*better to check if file exist otherwise just exit*/
		/*with messages                                    */
		
		
		fprintf(stderr, "Going to read data from %s\n", config_file);
	
        if (!Read_Config_File(config_file))
        {
            free(config_file);
            config_file = malloc( 25 );
            if (config_file == NULL)
                errH(1, 1, "malloc");

            sprintf(config_file, "/etc/wmsolatrc");
            fprintf(stderr, "Using %s as config file.\n", config_file);
        
			if (!Read_Config_File(config_file)){
				ShowError("Config file is not available!\n");
				}
           		free(config_file);
        }
		else {
		
	    	fprintf(stderr, "Using %s as config file.\n",config_file);		
			free(config_file);
		}
		
	fprintf(stderr, "latitude degrees = %d\n", latd);
	fprintf(stderr, "latitude minutes = %d\n", latm);
	fprintf(stderr, "latitude position = %c\n", *latc);
		
	if (latd<0 ||latd>90 || latm<0 || latm>59 ||
	   (*latc != 'N' && *latc != 'n' && *latc != 'S' && *latc != 's')){
		ShowError("Illegal data for geographical latitude!");
	}

	fprintf(stderr, "longitude degrees = %d\n", longd);
	fprintf(stderr, "longitude minutes = %d\n", longm);
	fprintf(stderr, "longitude position = %c\n", *longc);	
			
	latIsS = *latc == 'S' || *latc == 's';
	if (longd<0 || longd>180 || longm<0 || longm > 59 || 
	   (*longc != 'E' && *longc != 'e' && *longc != 'W' && *longc != 'w'))
		ShowError("Illegal data for geographical longitude!");

	longIsW = *longc == 'W' || *longc == 'w';

	if (timeZone <= -12.0 || timeZone > 12.0) ShowError("Illegal data for time zone!"); 

	if (*dlc != 'Y' && *dlc != 'y' && *dlc != 'N' && *dlc != 'n' && *dlc != '1' && *dlc != '0')
		ShowError("Illegal data for daylight saving time!");

	hasDayLt = *dlc == 'Y' || *dlc == 'y' || *dlc == '1';
	latitude = Degree2Radian(DegreeMinute2Degree(latd,latm));
	if (latIsS) latitude = - latitude;
	longitude = Degree2Radian(DegreeMinute2Degree(longd,longm));
	if (!longIsW) longitude = - longitude;
		
	MakeSchedule(day,month,year); 
	       
}

/*
 * Y a un probleme ?
 */

void ShowError( char *s) {
	fprintf(stderr, "Irrecoverable error, wmsolat exiting.\n");
	fprintf(stderr, "%s\n", s);
	exit(1);
}

/*
 * Bah c'est un parser!
 */ 
//	if (ParseLine(envv, enva, 9) != 9) ShowError("parameter(s) missing.");
//int ParseLine( char *line, char *argv[], int maxargv) { 

//	int argc=0;
//	char ch;
//			//  < 9
//	while (argc <maxargv) {
//		while ((ch = *line) != '\0' && (ch == ' ' || ch == '\t'))
//			line++;
//		if (ch == '\0') break;
//		argv[argc++] = line;
//		if (argc==1) {
//			while ((ch = *line) != '\0' && (ch < '0' || ch > '9'))
//				line++; line--;
//		} else
//			while ((ch = *line) != '\0' && ch != ' ' && ch != '\t')
//				line++;
//		if (ch == '\0') break;
//		*line = '\0'; line++;
//	}
//	return(argc);
//}

/*
 * Nombre de jours dans l'annee
 */

short DayNum(short d, short m, short y) {

	register short  i, dn;

	i=1; dn= --d;
	while (i<m) dn += NDays(i++,y);
	return(dn);
}

/*
 *
 */

void SetDow(short y, short m, short d) {

	stdow = ((long) (JulianDay(y,m,d, 0.0) + 1.5)) % 7;
}

/*
 * C'est parti pour le calcul des horaires du jour
 */

void HSchedule(short m, short y) {

	SDATE fd, ld;
	short   cd, cm, NDays0, NDays1, sd=1; 
	
	fd = *GDate(y,m,1);  
	ld = *GDate(y, m+1, 1);
	ld = *CalculDate(JulianDay(ld.year, ld.mon, ld.day, 0.0) - 1.0);
	
	NDays0 = DayNum(fd.day, fd.mon, fd.year);
	SetDow(fd.year,fd.mon,fd.day);
	cd = fd.day; cm = fd.mon;
	if (fd.mon != ld.mon) {        
		NDays1 = DayNum(NDays(fd.mon,fd.year), fd.mon, fd.year) +1;
		ComputeHours(fd.year,NDays0,NDays1);
		SetPrayTimeList(NDays0, NDays1,sd,cd,cm); 
		sd = NDays1 - NDays0 + 1;
		NDays0 = DayNum(1, ld.mon, ld.year);
		cd = 1; cm = ld.mon;
	}
	NDays1 = DayNum(ld.day, ld.mon, ld.year) + 1;
	ComputeHours(ld.year,NDays0,NDays1);
	SetPrayTimeList(NDays0,NDays1, sd, cd, cm); 
}

void MakeSchedule(short day, short month, short year) {

	short   NDays0, NDays1;

	Qiblah(day,month,year);
	NDays0 = DayNum(day,month,year); 
	NDays1 = NDays0 +1;
	ComputeHours(year,NDays0,NDays1);
	SetDow(year,month,day);
	SetPrayTimeList(NDays0,NDays1, hflag ? hd.day : day,day,month);
}

/*
 * Calcule le temps pour un nombre de jours X d'une annee
 */

void ComputeHours(short year, short first, short last) {
 
	double  coaltn, time0[6], coalt[6];
	register double t;
	register short  i,k,l;
    
	ComputeConstants(year==0 ? 1994 : year);  
	DayLight(year, &leap, hasDayLt, &beginDayLight, &endDayLight);
	time0[0] = 4.0;
	time0[1] = 6.0;
	time0[3] = 15.0;
	time0[4] = 18.0;
	time0[5] = 20.0;
	coalt[0] = Degree2Radian((double)(90+fajrDepr));
	coalt[1] = Degree2Radian(90.83);
	coalt[4] = coalt[1];
	coalt[5] = coalt[0];
	t = NoonTime(first,&coaltn);
	coalt[3] = atan(asrShadowRatio+tan(coaltn));
	time0[1] = (((t = Tempus(first,coalt[1],time0[1])) < 24.0) ? t : 6.0);  
	time0[3] = (((t = Tempus(first,coalt[3],time0[3])) < 24.0) ? t : 15.0);  
	time0[4] = (((t = Tempus(first,coalt[4],time0[4])) < 24.0) ? t : 18.0);
	if (fajrByInterval) {
		time0[0] = time0[1] - fajrInterval;  
		time0[5] = time0[4] + fajrInterval;
	} else {  
		time0[0] = (((t = Tempus(first,coalt[0],time0[0])) < 24.0) ? t : 4.0);  
		time0[5] = (((t = Tempus(first,coalt[5],time0[5])) < 24.0) ? t : 20.0);
	}         
	for (l=first, i=1; l<last; i++, l++) {
		k = l;  
		if (l>59 && year==0) k = l-1;
		tim[l][2] = NoonTime(k+1,&coaltn);
		coalt[3] = atan(asrShadowRatio+tan(coaltn));
		time0[1] = (((tim[l][1] = t = Tempus(k+1,coalt[1],time0[1]))
			    < 24.0) ? t : 6.0);
		time0[3] = (((tim[l][3] = t = Tempus(k+1,coalt[3],time0[3]))
					< 24.0) ? t : 15.0);  
		time0[4] = (((tim[l][4] = t = Tempus(k+1,coalt[4],time0[4]))
					< 24.0) ? t : 18.0);
		if (fajrByInterval) {
			tim[l][0] = time0[0] = time0[1] - fajrInterval;  
			tim[l][5] = time0[5] = time0[4] + fajrInterval;
		} else {  
			time0[0] = (((tim[l][0] = t = Tempus(k+1,coalt[0],time0[0]))
					< 24.0)  ? t : 4.0);
			time0[5] = (((tim[l][5] = t = Tempus(k+1,coalt[5],time0[5]))
					< 24.0) ? t : 20.0);
		}         
	}
	if (endDayLight !=0 && first<endDayLight) {
		i = beginDayLight-1;
		if (first>i) i=first;
		while (i<endDayLight && i<last) {
			for (k=0; k<6; k++) tim[i][k] += 1.0;
			i++;
		}
	}
}

/*
 * Calcul 'astral' du temps pour le 1er jours d'une annee
 */

void ComputeConstants( short year) {

	register double         t;
	long                    NDays;
	double                  obl, eccy;
	
	NDays = ((long) (year-1900))*365+(year-1901)/4;
	t = (NDays-0.5)/36525.0;
	obl = Degree2Radian(DegreeMinuteSecond2Degree(23L,27,8.26)-DegreeMinuteSecond2Degree(0L,0,46.845)*t);
	cosobl = cos(obl);
	sinobl = sin(obl);
	eccy = 0.01675104-4.180e-5*t-1.26e-7*t*t;
	perigee = Degree2Radian(FMOD(DegreeMinuteSecond2Degree(281L,13,15.0)+DegreeMinuteSecond2Degree(1L,43,9.03)*t+DegreeMinuteSecond2Degree(0L,0,1.63)*t*t,360.0));
	dmanom = Degree2Radian(DegreeMinuteSecond2Degree(35999L,2,59.10)/36525.0);
	anom0 = Degree2Radian(FMOD(DegreeMinuteSecond2Degree(358L,28,33.0)-DegreeMinuteSecond2Degree(0L,0,0.54)*t*t+FMOD(DegreeMinuteSecond2Degree(35999L,2,59.10)*t,360.0),360.0));
	delsid = HourMinuteSecond2Hour(2400,3,4.542)/36525.0;
	sidtm0 = FMOD(HourMinuteSecond2Hour(6,38,45.836)+FMOD(HourMinuteSecond2Hour(2400,3,4.542)*t,24.0),24.0);
	c1 = eccy*(2-eccy*eccy/4);
	c2 = 5*eccy*eccy/4;
}

/*
 * Annee Bissextile ?
 */

void DayLight(short year, short* leap, short hasDayLt, short* begin, short* finish) {

	short m4, m1, jan0, napr1, noct31, apr1, oct31;
	
	m4 = year%400;
	m1 = year%100;
	*leap = (year%4 == 0 && m1 != 0) || m4 == 0;
	if (hasDayLt==0) {
		*begin = 367;
		*finish = 0;
		return;
	}
	if (year==0) {
		*begin = 92; 
		*finish = *begin+213;
		return;
	}
	jan0 = (m4/100*124+1+m1+m1/4-*leap) % 7;
	napr1 = 91 + *leap; 
	noct31 = 304 + *leap;
	apr1 = (napr1+jan0) % 7;
	oct31 = (noct31+jan0) % 7;
	*begin = napr1+2-apr1;
	if ( *begin < napr1 ) *begin += 7;
	*finish = noct31+2-oct31;
	if ( *finish > noct31 ) *finish -= 7;
	*finish = *finish-1;
}

/*
 * Retourne l'heure exact de 'Midi au soleil'
 */
double NoonTime( short nday, double* coaltn) {

	register double t;
	double longh, days, anomaly, slong, sinslong, ra, decl, locmt;
	
	longh = longitude*HPR;
	days = nday+(12.0+longh)/24.0;
	anomaly = anom0+dmanom*days;
	slong = perigee+anomaly+c1*sin(anomaly)+c2*sin(anomaly*2);
	sinslong = sin(slong);
	ra = atan2(cosobl*sinslong,cos(slong))*HPR;
	if (ra<0.0) ra += 24.0;
	decl = asin(sinobl*sinslong);
	locmt = ra-delsid*days-sidtm0;
	t = locmt+longh+timeZone;
	if (t<0.0) t += 24.0;
	if (t>24.0) t -= 24.0;
	*coaltn = FABS(latitude-decl);
	return(t);
}

/*
 * L'heure par rapport a la position du soleil
 */

double Tempus(short nday, double coalt, double time0) {

	double longh, days, anomaly, slong, sinslong, ra, sindcl, cosha, ha, locmt;
	register double t;
	
	longh = longitude*HPR;
	days = nday+(time0+longh)/24.0;
	anomaly = anom0+dmanom*days;
	slong = perigee+anomaly+c1*sin(anomaly)+c2*sin(anomaly*2);
	sinslong = sin(slong);
	ra = atan2(cosobl*sinslong,cos(slong))*HPR;
	if (ra<0.0) ra += 24.0;
	sindcl = sinobl*sinslong;
	cosha = (cos(coalt)-sindcl*sin(latitude))/(sqrt(1.0-sindcl*sindcl)*cos(latitude));
	if (FABS(cosha)>1.0) return(1.0e7);
	ha = acos(cosha)*HPR;
	if (time0<12.0) ha = 24.0-ha;
	locmt = ha+ra-delsid*days-sidtm0;
	t = locmt+longh+timeZone;
	if (t<0.0) t += 24.0;
	if (t>24.0) t -= 24.0;
	return(t);
}

/*
 * Retourne la direction a prendre pour prier. Mais mais ou est la Mecca !?
 */

void Qiblah(short day, short month, short year) {

	double direc;
	double lat0 = 0.3739077, long0 = -0.69504828, dflong;

	dflong = longitude-long0;
	direc = (atan2(sin(dflong), cos(latitude)*tan(lat0)-sin(latitude)*cos(dflong))) * DPR;
	Degree2DegreeMinute(direc, &qiblah_d, &qiblah_m);
}

/*
 * Retourne la liste des heures de priere
 */
void SetPrayTimeList(short first, short last, short startdate, short cd, short cm) {

	double    t;
	register short  i,j,l;

	for (l=first, i=startdate; l<last; l++, i++) {
		for (j=0; j<6; j++) {
			t = tim[l][j];
			list_praying_hour[j] = t;  list_praying_minute[j] = 60.0*(t-list_praying_hour[j])+0.5;
			if (list_praying_minute[j]>=60) {
				list_praying_minute[j] = 0;
				list_praying_hour[j] += 1;
			}
		}
	}
}

/*
 * Conversions diverses 
 */

double DegreeMinuteSecond2Degree( long degree, short min, double sec ) { return((double) degree + min/60.0 + sec/3600.0); }
double HourMinuteSecond2Hour( short hour, short min, double sec ) { return((double) hour + min/60.0 + sec/3600.0); }
double Degree2Radian( double degree ) { return(degree*RPD); }
double DegreeMinute2Degree( short degree, short minute ) { return((double) degree + minute/60.0); }

void Degree2DegreeMinute( double degree, short *deg, short *min ) {

	double dabs;

	dabs = FABS(degree);
	*deg = dabs;
	*min = 60.0*(dabs - *deg) +0.5;
	if (*min>=60) {
		*min = 0;
		*deg += 1;
	}
}


/*
 * Retourne un etat suivant le quartier de lune 
 */

double MoonPhase(long  n, int nph) {

	double jd, t, t2, t3, k, ma, sa, tf, xtra;
	k = n + nph/4.0;  t = k/1236.85;  t2 = t*t; t3 = t2*t;
	jd =  2415020.75933 + 29.53058868*k - 1.178e-4 * t2
	    - 1.55e-7 * t3 
	    + 3.3e-4 * sin (RPD * (166.56 +132.87*t -0.009173*t2));

	sa =  RPD * (359.2242 + 29.10535608*k - 3.33e-5 * t2 - 3.47e-6 * t3);
	ma =  RPD * (306.0253 + 385.81691806*k + 0.0107306*t2 +1.236e-5 *t3);
	tf = RPD * 2.0 * (21.2964 + 390.67050646*k -0.0016528*t2 - 2.39e-6 * t3);

	if (nph==0 || nph==2)
		xtra = (0.1734 - 0.000393*t) * sin(sa)
		      +0.0021*sin(sa*2)
		      -0.4068*sin(ma) +0.0161*sin(2*ma) -0.0004*sin(3*ma)
		      +0.0104*sin(tf)
		      -0.0051*sin(sa+ma) -0.0074*sin(sa-ma)
		      +0.0004*sin(tf+sa) -0.0004*sin(tf-sa)
		      -0.0006*sin(tf+ma) +0.0010*sin(tf-ma)
		      +0.0005*sin(sa+ 2*ma);
	else if (nph==1 || nph==3) {
		xtra = (0.1721 - 0.0004*t) * sin(sa)
		      +0.0021*sin(sa*2)
		      -0.6280*sin(ma) +0.0089*sin(2*ma) -0.0004*sin(3*ma)
		      +0.0079*sin(tf)
		      -0.0119*sin(sa+ma) -0.0047*sin(sa-ma)
		      +0.0003*sin(tf+sa) -0.0004*sin(tf-sa)
		      -0.0006*sin(tf+ma) +0.0021*sin(tf-ma)
		      +0.0003*sin(sa+ 2*ma) +0.0004*sin(sa-2*ma)
		      -0.0003*sin(2*sa+ma);
		if (nph==1)
			xtra = xtra +0.0028 -0.0004*cos(sa) +0.0003*cos(ma);
		else
			xtra = xtra -0.0028 +0.0004*cos(sa) -0.0003*cos(ma);
	} else {
		printf("moon phase: illegal phase number\n");
		exit(1);
	}
	jd += xtra - (0.41 +1.2053*t +0.4992*t2)/1440;
	return (jd);
}

/*
 * Pour une nouvelle lune le meme jour
 */

double Visible(long n, double *rjd) {

	double jd;
	float tf;
	long d;

	jd = MoonPhase( n, 0);
	*rjd = jd;
	d = jd;
	tf = (jd - d);
	if (tf<=0.5) return(jd+1.0);
	tf = (tf-0.5)*24 +TIMZ; 
	if (tf>TIMDIF) return(jd+1.0);
	return(jd);
}

/* 
 * Calendrier
 */

double JulianDay( int year, int month, int day, float time) {

	double jul;
	long m,y,ja;

	if (year<0) year++;
	if (month>2) {
		y = year; m =month;
	} else {
		y = year-1; m =month+12;
	}
	jul = ((double)y) * 365.25;
	if (y<1) jul -= 0.75;
	jul = ((long) jul) + ( (long) (30.6001*(m+1))) +day +time +1720994.5;
	if (year +month*1e-2 +(day+time)*1e-4 >= 1582.1015) {
		ja = 0.01*y;
		jul = jul +2 -ja + ((long) (0.25*ja));
	}
	return jul;
}

/*
 * Inverse de JulianDay
 */

SDATE *CalculDate(double julday ) {

	static SDATE sd;
	long	z, alpha, a, b, c, d, e;
	double	f;

	julday += 0.5; z = julday; f = julday-z;
	if ( z<2299161L )
		a = z;
	else {
		alpha = (z-1867216.25) / 36524.25;
		a     = z + 1 +alpha - alpha / 4;
	}
	b = a + 1524;  c = (b - 122.1) / 365.25; d = 365.25 * c;
	e = (b-d) / 30.6001;
	f += b -d - ((long) (30.6001*e));
	sd.day = f;
	sd.time = f-sd.day;
	sd.mon = (e>13) ? e-13 : e-1;
	sd.year = (sd.mon>2) ? c-4716 : c-4715;
	sd.dw = ((long) (julday -sd.time + 1.1)) % 7;
	if ( sd.year<=0) sd.year--;
	return(&sd);
}

/*
 * Conversion de date (georgian/julian)
 */

SDATE *HDate (int y, int m, int d) {

	static SDATE h;
	double jd, mjd, rjd;
	long k, hm;

	jd = JulianDay(y,m,d,0.00); 
	k = 0.6 + (y + ((int) (m-0.5)) /12.0 + d/365.0 - 1900) *12.3685;
	do  {mjd = Visible(k--, &rjd);} while (mjd>jd);  k++;
	hm = k -1048;  
	h.year = 1405 + (hm / 12);
	h.mon =  (hm % 12) +1; 
	if (hm !=0 && h.mon<=0) {h.mon +=12; h.year--; }
	if (h.year<=0) h.year--;
	h.day = jd -mjd +1.0;
	h.time = 0.5;
	h.dw = ((long) (jd+1.5)) % 7;
	return(&h);
}

/*
 * Retourne la date 'Hijri'
 */

SDATE *GDate( int y, int m, int d) {

	double jd, rjd;
	long k;
	SDATE *sd;

	if (y<0) y++;
	k = m+ y*12 - NMONTHS;
	jd = Visible(k+1048L, &rjd) +d;
	sd = CalculDate(jd);
	sd->nmtime = rjd;
	return (sd);
}

/*
 * Combien de jour dans le mois
 */
int NDays(int m, int y) {

	static short  ndmnth[13] = {0,31,28,31,30,31,30,31,31,30,31,30,31};

	int nd = ndmnth[m];
	if (m==2 && (y%4==0 && (y<1582 || y%100!=0 || y%400==0))) nd++;
	return nd;
}

/*****************************************************************************\
|* ReadConfigString   from wmtz :)                                                   *|
\*****************************************************************************/
int ReadConfigString(FILE *fp, char *setting, char *value)
{   
    char str[1024];
    char buf[1024];
    int i;
    int len;
    int slen;
    char *p=NULL;

    if (!fp) // nofile?
    {
        return 0;
    }

    snprintf(str,1024, "%s=", setting); // setting to str
    slen = strlen(str); //get length
    
    fseek(fp, 0, SEEK_SET); // set to beginning of file

    while ( !feof(fp) )  // not eof
    {
        
        if (!fgets(buf, 512, fp))   //cannot gets 512
	  break;
        
        len = strlen(buf);

        /* strip linefeed */
        for (i=0; i!=len; i++)
	  if (buf[i] == '\n')
	    buf[i] = 0;

        /* printf("Scanning '%s'...\n", buf); */
        if ( strncmp(buf, str, strlen(str)) == 0)
        {
            /* found our setting */
	  
	  for(i=0; i!=slen; i++)
	    if ( buf[i] == '=' )
	      {
		p=buf+i+1;
		strcpy(value, p); // returning value as character
		return 1;
	      }
        }
    }    
    return 0;
}


/*****************************************************************************\
|* ReadConfigInt                                                             *|
\*****************************************************************************/
int ReadConfigInt(FILE *fp, char *setting, int *value)
{
    char buf[1024];

    if (ReadConfigString(fp, setting, (char *) &buf))
    {
        *value = atoi(buf);
        return 1;
    }

    return 0;
}


/*****************************************************************************\
|* ReadConfigDouble                                                          *|
\*****************************************************************************/
int ReadConfigDouble(FILE *fp, char *setting, double *value)
{
    char buf[1024];

    if (ReadConfigString(fp, setting, (char *) &buf))
    {
        *value = atof(buf);
        return 1;
    }

    return 0;
}


/*****************************************************************************\
|* Read_Config_File                                                          *|
\*****************************************************************************/
int Read_Config_File( char *filename )
{
    FILE *fp;
    char temp[253];
  
    fp = fopen(filename, "r"); /*open read only*/
    if (fp)
    {
	        ReadConfigString(fp, "LatitudeDegrees", temp);
		/*fprintf(stderr, "latd=%s\n", temp);*/
		latd = atoi(temp);
		ReadConfigString(fp, "LatitudeMinutes", temp);
		/*fprintf(stderr, "latm=%s\n", temp);	*/
		latm = atoi(temp);
   		
		ReadConfigString(fp, "LatitudePosition", temp);
		latc = "N"; // initialize to North change if South
		if (strcmp (temp, "N") != 0 && strcmp (temp, "n") != 0) latc = "S";

   		ReadConfigString(fp, "LongitudeDegrees", temp);
		longd = atoi(temp);
		ReadConfigString(fp, "LongitudeMinutes", temp);
		longm = atoi(temp);
		longc = "E"; // initialize to East
		ReadConfigString(fp, "LongitudePosition", temp);
		if (strcmp (temp, "E") != 0 && strcmp (temp, "e") != 0) longc = "W";

		/*fprintf(stderr, "longc=%s\n", temp);*/
		ReadConfigString(fp, "TimeZone", temp);
		timeZone = atoi(temp);
		dlc = "N"; // initialize to no daylight
    		ReadConfigString(fp, "DaylightSaving", temp);
		if (temp == "Y" || temp == "y") dlc = "Y";

        	ReadConfigString(fp, "ScriptFileAzan", scriptFileAzan);
		ReadConfigString(fp, "ScriptFileOpen", scriptFileOpen);
		ReadConfigString(fp, "ScriptFileRemind", scriptFileRemind);
		ReadConfigString(fp, "ReminderMinutes", temp);
		reminder_minutes = atoi(temp);


		fclose(fp);
        return 1;
    }
    else
    {
        errH(1, 0, "Unable to open %s", filename);
        return 0;
    }
}

/*****************************************************************************\
|* errH                                                                      *|
\*****************************************************************************/
void errH(int printErrno, int exitCode, const char *msg, ...)
{
    int error = errno;
    va_list arg;
    char buf[2048];

    /* Put the name of the program first */
    buf[0] = 0;
    strcat(buf, pname);
    strcat(buf, ": ");
    
    va_start(arg, msg);
    vsprintf(buf+strlen(buf), msg, arg);
    if (printErrno)
        sprintf(buf+strlen(buf), ": %s", strerror(error));
    strcat(buf, "\n");
    fflush(stdout); 
    fputs(buf, stderr);
    fflush(NULL); 
    va_end(arg);

    if ( exitCode )
         exit(exitCode);

    return;
}
